import { ComputedRef, Ref, ComponentInternalInstance, VNodeProps, AllowedComponentProps, ComponentCustomProps, Slot, ComponentPublicInstance, ComponentOptionsBase, VNode, RendererNode, RendererElement, ComponentOptionsMixin, ReactiveEffect, nextTick, WatchOptions, WatchStopHandle, ShallowUnwrapRef, ComponentCustomProperties } from 'vue';

interface ValidationResult {
    errors: string[];
}
interface FormController {
    register(field: any): void;
    unregister(field: any): void;
    values: Record<string, any>;
    fields: ComputedRef<Record<string, any>>;
    schema?: Record<string, GenericValidateFunction | string | Record<string, any>>;
    validateSchema?: (shouldMutate?: boolean) => Promise<Record<string, ValidationResult>>;
    setFieldValue: (path: string, value: any) => void;
}
declare type MaybeReactive<T> = Ref<T> | ComputedRef<T> | T;
declare type SubmitEvent = Event & {
    target: HTMLFormElement;
};
declare type SubmissionHandler = (values: Record<string, any>, evt?: SubmitEvent) => any;
declare type GenericValidateFunction = (value: any) => boolean | string | Promise<boolean | string>;

interface ValidationOptions {
    name?: string;
    values?: Record<string, any>;
    bails?: boolean;
    skipIfEmpty?: boolean;
    isInitial?: boolean;
}
/**
 * Validates a value against the rules.
 */
declare function validate(value: any, rules: string | Record<string, any> | GenericValidateFunction, options?: ValidationOptions): Promise<ValidationResult>;

interface FieldContext {
    field: string;
    value: any;
    form: Record<string, any>;
    rule: {
        name: string;
        params?: Record<string, any> | any[];
    };
}
declare type ValidationRuleFunction = (value: any, params: any[] | Record<string, any>, ctx: FieldContext) => boolean | string | Promise<boolean | string>;
declare type ValidationMessageGenerator = (ctx: FieldContext) => string;

/**
 * Adds a custom validator to the list of validation rules.
 */
declare function defineRule(id: string, validator: ValidationRuleFunction): void;

interface VeeValidateConfig {
    bails: boolean;
    generateMessage: ValidationMessageGenerator;
}
declare const configure: (cfg: Partial<VeeValidateConfig>) => void;

declare const Field: {
    new (...args: any[]): {
        $: ComponentInternalInstance;
        $data: {};
        $props: {
            name: string;
        } & {
            disabled?: boolean;
            bails?: boolean;
            immediate?: boolean;
            as?: any;
            rules?: any;
        } & VNodeProps & AllowedComponentProps & ComponentCustomProps;
        $attrs: Record<string, unknown>;
        $refs: Record<string, unknown>;
        $slots: Readonly<{
            [name: string]: Slot;
        }>;
        $root: ComponentPublicInstance<{}, {}, {}, {}, {}, {}, {}, ComponentOptionsBase<any, any, any, any, any, any, any, any, string>>;
        $parent: ComponentPublicInstance<{}, {}, {}, {}, {}, {}, {}, ComponentOptionsBase<any, any, any, any, any, any, any, any, string>>;
        $emit: (event: string, ...args: any[]) => void;
        $el: any;
        $options: ComponentOptionsBase<{
            name: string;
        } & {
            disabled?: boolean;
            bails?: boolean;
            immediate?: boolean;
            as?: any;
            rules?: any;
        }, () => VNode<RendererNode, RendererElement, {
            [key: string]: any;
        }> | VNode<RendererNode, RendererElement, {
            [key: string]: any;
        }>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string>;
        $forceUpdate: ReactiveEffect<any>;
        $nextTick: typeof nextTick;
        $watch: (this: ComponentInternalInstance, source: TimerHandler, cb: Function, options?: WatchOptions<boolean>) => WatchStopHandle;
    } & {
        name: string;
    } & {
        disabled?: boolean;
        bails?: boolean;
        immediate?: boolean;
        as?: any;
        rules?: any;
    } & ShallowUnwrapRef<() => VNode<RendererNode, RendererElement, {
        [key: string]: any;
    }> | VNode<RendererNode, RendererElement, {
        [key: string]: any;
    }>[]> & {} & ComponentCustomProperties & Readonly<{
        disabled: boolean;
        name: string;
        bails: boolean;
        immediate: boolean;
        as: any;
        rules: any;
    } & {}>;
    __isFragment?: never;
    __isTeleport?: never;
    __isSuspense?: never;
} & ComponentOptionsBase<Readonly<{
    disabled: boolean;
    name: string;
    bails: boolean;
    immediate: boolean;
    as: any;
    rules: any;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string> & {
    props: {
        as: {
            type: (ObjectConstructor | StringConstructor)[];
            default: any;
        };
        name: {
            type: StringConstructor;
            required: true;
        };
        rules: {
            type: (ObjectConstructor | FunctionConstructor | StringConstructor)[];
            default: any;
        };
        immediate: {
            type: BooleanConstructor;
            default: boolean;
        };
        bails: {
            type: BooleanConstructor;
            default: () => boolean;
        };
        disabled: {
            type: BooleanConstructor;
            default: boolean;
        };
    } & ThisType<void>;
} & ThisType<ComponentPublicInstance<Readonly<{
    disabled: boolean;
    name: string;
    bails: boolean;
    immediate: boolean;
    as: any;
    rules: any;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, Readonly<{
    disabled: boolean;
    name: string;
    bails: boolean;
    immediate: boolean;
    as: any;
    rules: any;
} & {}>, ComponentOptionsBase<Readonly<{
    disabled: boolean;
    name: string;
    bails: boolean;
    immediate: boolean;
    as: any;
    rules: any;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string>>>;

declare const Form: {
    new (...args: any[]): {
        $: ComponentInternalInstance;
        $data: {};
        $props: {} & {
            as?: string;
            validationSchema?: Record<string, any>;
            initialValues?: Record<string, any>;
        } & VNodeProps & AllowedComponentProps & ComponentCustomProps;
        $attrs: Record<string, unknown>;
        $refs: Record<string, unknown>;
        $slots: Readonly<{
            [name: string]: Slot;
        }>;
        $root: ComponentPublicInstance<{}, {}, {}, {}, {}, {}, {}, ComponentOptionsBase<any, any, any, any, any, any, any, any, string>>;
        $parent: ComponentPublicInstance<{}, {}, {}, {}, {}, {}, {}, ComponentOptionsBase<any, any, any, any, any, any, any, any, string>>;
        $emit: (event: string, ...args: any[]) => void;
        $el: any;
        $options: ComponentOptionsBase<{} & {
            as?: string;
            validationSchema?: Record<string, any>;
            initialValues?: Record<string, any>;
        }, () => VNode<RendererNode, RendererElement, {
            [key: string]: any;
        }> | VNode<RendererNode, RendererElement, {
            [key: string]: any;
        }>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string>;
        $forceUpdate: ReactiveEffect<any>;
        $nextTick: typeof nextTick;
        $watch: (this: ComponentInternalInstance, source: TimerHandler, cb: Function, options?: WatchOptions<boolean>) => WatchStopHandle;
    } & {} & {
        as?: string;
        validationSchema?: Record<string, any>;
        initialValues?: Record<string, any>;
    } & ShallowUnwrapRef<() => VNode<RendererNode, RendererElement, {
        [key: string]: any;
    }> | VNode<RendererNode, RendererElement, {
        [key: string]: any;
    }>[]> & {} & ComponentCustomProperties & Readonly<{
        as: string;
        validationSchema: Record<string, any>;
        initialValues: Record<string, any>;
    } & {}>;
    __isFragment?: never;
    __isTeleport?: never;
    __isSuspense?: never;
} & ComponentOptionsBase<Readonly<{
    as: string;
    validationSchema: Record<string, any>;
    initialValues: Record<string, any>;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string> & {
    props: {
        as: {
            type: StringConstructor;
            default: string;
        };
        validationSchema: {
            type: ObjectConstructor;
            default: any;
        };
        initialValues: {
            type: ObjectConstructor;
            default: any;
        };
    } & ThisType<void>;
} & ThisType<ComponentPublicInstance<Readonly<{
    as: string;
    validationSchema: Record<string, any>;
    initialValues: Record<string, any>;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, Readonly<{
    as: string;
    validationSchema: Record<string, any>;
    initialValues: Record<string, any>;
} & {}>, ComponentOptionsBase<Readonly<{
    as: string;
    validationSchema: Record<string, any>;
    initialValues: Record<string, any>;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string>>>;

declare const ErrorMessage: {
    new (...args: any[]): {
        $: ComponentInternalInstance;
        $data: {};
        $props: {
            name: string;
        } & {
            as?: string;
        } & VNodeProps & AllowedComponentProps & ComponentCustomProps;
        $attrs: Record<string, unknown>;
        $refs: Record<string, unknown>;
        $slots: Readonly<{
            [name: string]: Slot;
        }>;
        $root: ComponentPublicInstance<{}, {}, {}, {}, {}, {}, {}, ComponentOptionsBase<any, any, any, any, any, any, any, any, string>>;
        $parent: ComponentPublicInstance<{}, {}, {}, {}, {}, {}, {}, ComponentOptionsBase<any, any, any, any, any, any, any, any, string>>;
        $emit: (event: string, ...args: any[]) => void;
        $el: any;
        $options: ComponentOptionsBase<{
            name: string;
        } & {
            as?: string;
        }, () => VNode<RendererNode, RendererElement, {
            [key: string]: any;
        }> | VNode<RendererNode, RendererElement, {
            [key: string]: any;
        }>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string>;
        $forceUpdate: ReactiveEffect<any>;
        $nextTick: typeof nextTick;
        $watch: (this: ComponentInternalInstance, source: TimerHandler, cb: Function, options?: WatchOptions<boolean>) => WatchStopHandle;
    } & {
        name: string;
    } & {
        as?: string;
    } & ShallowUnwrapRef<() => VNode<RendererNode, RendererElement, {
        [key: string]: any;
    }> | VNode<RendererNode, RendererElement, {
        [key: string]: any;
    }>[]> & {} & ComponentCustomProperties & Readonly<{
        name: string;
        as: string;
    } & {}>;
    __isFragment?: never;
    __isTeleport?: never;
    __isSuspense?: never;
} & ComponentOptionsBase<Readonly<{
    name: string;
    as: string;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string> & {
    props: {
        as: {
            type: StringConstructor;
            default: any;
        };
        name: {
            type: StringConstructor;
            required: true;
        };
    } & ThisType<void>;
} & ThisType<ComponentPublicInstance<Readonly<{
    name: string;
    as: string;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], {}, {}, {}, Record<string, any>, Readonly<{
    name: string;
    as: string;
} & {}>, ComponentOptionsBase<Readonly<{
    name: string;
    as: string;
} & {}>, () => VNode<RendererNode, RendererElement, {
    [key: string]: any;
}> | VNode<RendererNode, RendererElement, {
    [key: string]: any;
}>[], unknown, {}, {}, ComponentOptionsMixin, ComponentOptionsMixin, Record<string, any>, string>>>;

interface FieldOptions {
    initialValue: any;
    disabled: MaybeReactive<boolean>;
    immediate?: boolean;
    bails?: boolean;
    form?: FormController;
    type?: string;
    valueProp?: MaybeReactive<any>;
}
declare type RuleExpression = MaybeReactive<string | Record<string, any> | GenericValidateFunction>;
/**
 * Creates a field composite.
 */
declare function useField(fieldName: MaybeReactive<string>, rules: RuleExpression, opts?: Partial<FieldOptions>): {
    name: MaybeReactive<string>;
    value: any;
    meta: {
        untouched: boolean;
        touched: boolean;
        dirty: boolean;
        pristine: boolean;
        valid: boolean;
        invalid: boolean;
        passed: boolean;
        failed: boolean;
        validated: boolean;
        pending: boolean;
        changed: boolean;
    };
    errors: Ref<string[]>;
    errorMessage: ComputedRef<string>;
    aria: ComputedRef<{
        'aria-invalid': string;
        'aria-describedBy': string;
    }>;
    reset: () => void;
    validate: () => Promise<ValidationResult>;
    handleChange: (e: Event) => void;
    onBlur: () => void;
    disabled: MaybeReactive<boolean>;
    setValidationState: (result: ValidationResult) => ValidationResult;
    type: string;
    valueProp: any;
    checked: ComputedRef<any>;
    idx: number;
};

interface FormOptions {
    validationSchema?: Record<string, GenericValidateFunction | string | Record<string, any>>;
    initialValues?: Record<string, any>;
}
declare function useForm(opts?: FormOptions): {
    errors: ComputedRef<any>;
    meta: ComputedRef<Record<string, boolean>>;
    form: FormController;
    values: ComputedRef<any>;
    validate: () => Promise<boolean>;
    isSubmitting: Ref<boolean>;
    handleReset: () => void;
    handleSubmit: (fn?: SubmissionHandler) => (e: unknown) => Promise<void>;
    submitForm: (e: unknown) => Promise<void>;
};

export { ErrorMessage, Field, Form, configure, defineRule, useField, useForm, validate };
